namespace Cable.Port.View {

    public interface Window : GLib.Object {

        public abstract bool welcome_shown { get; set; }
        public abstract Gee.ArrayList <Server> servers { owned get; }
        public abstract Server? current_server { get; }
        public abstract Chat? current_chat { get; set; }
        public abstract string title { get; set; }

        public string[] server_ids {
            owned get {
                string[] result = {};
                foreach (var server in servers)
                    result += server.id;
                return result;
            }
        }

        public uint n_servers {
            get { return servers.size; }
        }

        public signal void remove_channel ();
        public signal void add_channel ();
        public signal void selected (Chat chat);
        public signal void show_preferences ();
        public signal void join (string address, string nick, string channel_name);
        public signal void close ();

        public abstract Server? add_server (string server_name, string server_id);
        public abstract void remove_server (string server_id);

        public bool has_server (string server_id) {
            return server_id in server_ids;
        }

        public Server? get_server_by_id (string server_id) {
            foreach (var server in servers)
                if (server.id == server_id)
                    return server;

            return (Server) null;
        }
    }

    public interface Server : GLib.Object {

        public abstract bool away { set; }
        public abstract string name { get; set; }
        public abstract string id { get; set; }

        public abstract Gee.ArrayList <Chat> chats { owned get; }

        public string[] chat_ids {
            owned get {
                string[] result = {};
                foreach (var chat in chats)
                    result += chat.id;
                return result;
            }
        }

        public abstract signal void toggle_away ();
        public abstract signal void part_all ();

        public abstract void add_channel (string chat_name, string chat_id);
        public abstract void add_private_chat (string chat_name, string chat_id);
        public abstract void remove_chat (string chat_id);

        public bool has_chat (string chat_id) {
            return chat_id in chat_ids;
        }

        public Chat? get_chat (string chat_id) {
            foreach (var chat in chats)
                if (chat.id == chat_id)
                    return chat;
            return (Chat) null;
        }
    }

    public interface Chat : GLib.Object {

        public abstract int unread_messages { get; set; }
        public abstract string nick { get; set; }
        public abstract string name { get; set; }
        public abstract string id { get; set; }
        public abstract string entry { get; set; }
        public abstract bool has_focus { get; }

        public signal void send_message (string message);
        public signal bool focused ();

        public abstract void rename_user (string? old_nick, string new_nick,
                                          UserType type, bool silent = false);
        public abstract void set_away_status (string nick, bool is_away);
        public abstract void message (owned string from, string message,
                                      MessageType type = MessageType.NORMAL);
    }

    public interface Channel : GLib.Object, Chat {

        public abstract State state { get; set; }
        public abstract string topic { get; set; }

        public signal void join ();
        public signal void part ();
        public signal void send_message (string message);
        public signal bool focused ();
        public signal void private_chat_requested (string name);

        public abstract bool has_user (string nick);
        public abstract void list_users (string[] nicks, string[] prefixes);
        public abstract void add_user (string nick, UserType type, bool silent = false);
        public abstract void remove_user (string nick, string message, bool silent = false);
    }

    public interface PrivateChat : GLib.Object, Chat {

        public signal void send_message (string message);
        public signal bool focused ();
    }
}