/***
    Copyright (C) 2013 Cable Developers

    This program or library is free software; you can redistribute it
    and/or modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 3 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General
    Public License along with this library; if not, write to the
    Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
    Boston, MA 02110-1301 USA.
***/

/**
 * This class represents a network.
 */
public class Cable.Utils.Network : GLib.Object {

    public string address { get; construct; }
    public string name { get; set; }
    public string password { get; set; }

    static Gee.ArrayList <Network> _all;
    public static Gee.List <Network> all {
        owned get {
            return _all.read_only_view as Gee.List;
        }
    }

    public signal void altered ();

    public Network (string address, string? name = null, string? password = null)
        requires (Network.search (address) == null) {

        GLib.Object (address: address,
                     name: name ?? address,
                     password: password ?? "");

        notify.connect (() => {
            Settings.preferences.networks = export ();
        });

        _all.add (this);
        
        //Settings.preferences.networks = export ();
    }

    ~Network () {
        _all.remove (this);
    }

    public static void init ()
        requires (_all == null) {
        
        _all = new Gee.ArrayList <Network> ();
        import (Settings.preferences.networks);
    }

    static void import (string[] data) {
        foreach (var network in data) {
            var values = network.split (":");

            switch (values.length) {
                case 0:
                    warn_if_reached ();
                    break;
                case 1:
                    new Network (values[0]);
                    break;
                case 2:
                    new Network (values[0], values[1]);
                    break;
                case 3:
                    new Network (values[0], values[1], values[2]);
                    break;
                default:
                    warn_if_reached ();
                    break;
            }
        }
    }
    
    static string[] export () {
        string[] result = {};
        
        foreach (var network in all)
            result += string.join (":", network.address, network.name, network.password);
        
        return result;
    }

    public static void remove (string address)
        requires (search (address) != null)
        ensures (search (address) == null) {
        
        _all.remove (search (address));
        Settings.preferences.networks = export ();
    }

    public static Network get_or_create (string address) {
        return search (address) ?? new Network (address);
    }

    /**
     * Check if this is a duplicate, and if so, return the original.
     */
    public static Network? search (string address) {
        foreach (var network in _all)
            if (network.address == address)
                return network;

        return (Network) null;
    }
}
